/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* Restricted Materials of IBM                                            */
/*                                                                        */
/* (C) COPYRIGHT International Business Machines Corp. 2012               */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
/* "@(#) 1.2 src/ppe/hpctq/source/src/mp_profiler/include/mpt.h, ppe.hpctq, ppe_rjunoq, rjunoqs001a 12/12/17 12:41:11" */
/*
 * 10/16/2008 caf 153883 - change return void *MT_get_calleraddress
 */
#ifndef _MPT_H
#define _MPT_H

#include <stdio.h>
#include <stdlib.h>

#define COUNTS          1000
#define BYTES           1001
#define COMMUNICATIONTIME 1002
#define STACK           1003
#define HEAP            1004
#define MAXSTACKFUNC    1005
#define ELAPSEDTIME     1006
#define AVGHOPS         1007

/*----------------------------------------------------------*/
/*    define a structure to hold summarized info            */
/*----------------------------------------------------------*/
struct MT_summarystruct {
        int min_rank;
        int max_rank;
        int med_rank;

        void *min_result;
        void *max_result;
        void *med_result;
        void *avg_result;
        void *sum_result;

        void *all_result;
        void *sorted_all_result;
        int *sorted_rank;

};


struct MT_tracebufferstruct {

	int number_events;
	double total_buffer; /* in terms of MBytes */
	double used_buffer;
	double free_buffer;
};



struct MT_envstruct {

        int mpirank;
#if defined(BGQ)
        int aCoord;
        int bCoord;
        int cCoord;
        int dCoord;
        int eCoord;
        int aSize;
        int bSize;
        int cSize;
        int dSize;
        int eSize;
#else
        int xCoord;
        int yCoord;
        int zCoord;
        int xSize;
        int ySize;
        int zSize;
#endif
        int procid;
        int ntasks;
        double clockHz;
        int nmpi;

};


struct MT_memorystruct {

        unsigned int max_stack_address;
        unsigned int min_stack_address;
        unsigned int max_heap_address;

};


struct MT_layoutstruct {
#if defined(BGQ)
        int *aCoord;
        int *bCoord;
        int *cCoord;
        int *dCoord;
        int *eCoord;
#else
        int *xCoord;
        int *yCoord;
        int *zCoord;
#endif
        int *procid;

};



#ifndef MT_CALLERSTRUCT
#define MT_CALLERSTRUCT
struct MT_callerstruct {

	char *filepath;
	char *filename;
	char *funcname;
	int lineno;
};
#endif

/*----------------------------------*/
/* functions called by the profiler */
/*----------------------------------*/
int MT_trace_event(int);
int MT_output_trace(int);
int MT_output_text(void);


/*----------------------------------*/
/* utility function prototype       */
/*----------------------------------*/

long long MT_get_mpi_counts(int);   	/* number of calls for a MPI */
double MT_get_mpi_bytes(int);       	/* size of data tranfer for a MPI */
double MT_get_mpi_time(int);		/* time used for a MPI */
double MT_get_time(void);		/* time from the MPI_Init */
double MT_get_elapsed_time(void);       /* time between MPI_Finalize and MPI_Init */

char *MT_get_mpi_name(int);		/* name for a MPI */


int MT_get_tracebufferinfo(struct MT_tracebufferstruct *); /* info for trace buffer */
int MT_get_memoryinfo(struct MT_memorystruct *);	/* info for the stack/heap */
void * MT_get_calleraddress(int level);	/* info for the caller */
int MT_get_callerinfo(void *, struct MT_callerstruct *);/* get caller detailed info */


int MT_get_allresults(int , int , struct MT_summarystruct *); /* get global info */

int MT_print_message_size_dist(FILE *);

void MT_trace_start(void);
void MT_trace_stop(void);

double MT_get_avg_hops(void);          /* average hops each MPI message travels */
void MT_get_environment(struct MT_envstruct *);           /* self id info e.g., mpi rank */
void MT_get_layout(struct MT_layoutstruct *);  
void MT_print_flat_profile(FILE *);
void MT_print_memory_stats(FILE *);

#endif /* define _MPT_H */
